/**
 *  Copyright (c) 1999-2010, Ecole des Mines de Nantes
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *      * Neither the name of the Ecole des Mines de Nantes nor the
 *        names of its contributors may be used to endorse or promote products
 *        derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package choco.kernel.common.util.iterators;

import java.util.Iterator;

public class AppendIterator<E> extends DisposableIterator<E> {

	private final Iterator<Iterator<? extends E>> master;

	private Iterator<? extends E> slave;

	@SuppressWarnings({"unchecked"})
    public AppendIterator(final Iterator<? extends E>... iterators) {
		super();
		master=ArrayIterator.getIterator(iterators);
	}

	/**
	 * @see java.util.Iterator#hasNext()
	 */
	@Override
	public boolean hasNext() {
		if(slave!=null && slave.hasNext()) {return true;}
		else {
			while(master.hasNext()) {
				slave=master.next();
				if(slave.hasNext()) {return true;}
			}
		}
		return false;
	}

	/**
	 * @see java.util.Iterator#next()
	 */
	@Override
	public E next() {
		if(slave==null ||
				( ! slave.hasNext() ) ) {
			while(master.hasNext()) {
				slave=master.next();
				if(slave.hasNext()) {break;}
			}
		}
		return slave.next();
	}

    /**
     * This method allows to declare that an object is not used anymore. It
     * can be reused by another object.
     */
    @Override
    public void dispose() {}
}
