/**
 *  Copyright (c) 1999-2010, Ecole des Mines de Nantes
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *      * Neither the name of the Ecole des Mines de Nantes nor the
 *        names of its contributors may be used to endorse or promote products
 *        derived from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package choco.kernel.solver.variables.scheduling;

import choco.kernel.common.util.tools.TaskUtils;
import choco.kernel.solver.ContradictionException;
import choco.kernel.solver.variables.integer.IntDomainVar;

public abstract class AbstractRTask implements IRTask {

		
	public final int taskIdx;

	public TaskVar taskvar;
	
	public IntDomainVar usage;
	
	public IntDomainVar height;
	
	protected int value;
	
	public AbstractRTask(int taskIdx) {
		super();
		this.taskIdx = taskIdx;
	}

	@Override
	public final int getMaxHeight() {
		return getHeight().getSup();
	}

	@Override
	public final int getMinHeight() {
		return getHeight().getInf();
	}

	@Override
	public final long getMaxConsumption() {
		return TaskUtils.getMaxConsumption(this);
	}

	@Override
	public final long getMinConsumption() {
		return TaskUtils.getMinConsumption(this);
	}

	@Override
	public final int getTaskIndex() {
		return taskIdx;
	}

	@Override
	public final int getStoredValue() {
		return value;
	}

	@Override
	public void storeValue(int val) {
		this.value=val;

	}

	@Override
	public final boolean updateECT() throws ContradictionException {
		return updateECT(value);
	}

	@Override
	public final boolean updateEST() throws ContradictionException {
		return updateEST(value);
	}

	@Override
	public final boolean updateLCT() throws ContradictionException {
		return updateLCT(value);
	}

	@Override
	public final boolean updateLST() throws ContradictionException {
		return updateLST(value);
	}




}
