package org.jcp.jsr331.samples;

//===============================================
//J A V A  C O M M U N I T Y  P R O C E S S
//
//J S R  3 3 1
//
//Test Compatibility Kit
//
//================================================

/***************************************************************************\
 * 
 * ------------  Knapsack Problem -----------
 * 
 * The knapsack problem or rucksack problem is a problem in combinatorial 
 * optimization: Given a set of items, each with a weight and a value, 
 * determine the number of each item to include in a collection so that 
 * the total weight is less than or equal to a given limit and the total 
 * value is as large as possible. It derives its name from the problem 
 * faced by someone who is constrained by a fixed-size knapsack and must 
 * fill it with the most useful items.
 *                                       -- Wikipedia
 * 
 * Knapsack Max Unit : 10
 * 
 * Items   Weight Value   Total Items
 * ---------------------------------
 * Gold    1      15       20
 * Silver  2      10       30
 * Bronze  3      5        40
 * 
 * 
 *      1G +  2S + 3B <= 25  // to find total Weight
 *     15G + 10S + 5B  =  P  // to find total Profit
 * 
 * 
 \***************************************************************************/

import javax.constraints.*;

public class Knapsack {
	
	Problem p = ProblemFactory.newProblem("Knapsack Problem");
	
	int itemSize[] = { 1, 2, 3 };
	int itemValue[] = { 15, 10, 5 };
	final int knapsackSize = 25;

	public void define() {

		// === Define Variable(s)
		Var G = p.variable("G", 0, 20);
		Var S = p.variable("S", 0, 30);
		Var B = p.variable("B", 0, 40);
		Var P = p.variable("P", 0, 1000);

		// === Post Constraint(s)
		// 1. --- 1G + 2S + 3B <= 25
		p.post(itemSize, new Var[] { G, S, B }, "<=", knapsackSize);

		// 2. --- 15G + 10S + 5B = P
		p.post(itemValue, new Var[] { G, S, B }, "=", P);

	}

	// === Problem Resolution
	public void solve() {
		// --- Get the Solver
		Solver solver = p.getSolver();

		// --- Find the Solution

		/*
		 * // --- Find all Solutions Solution[] solutions =
		 * solver.findAllSolutions(); for (int i = 0; i < solutions.length; i++)
		 * { solutions[i].log(); }
		 */

		// --- Find Exact Solution
		Solution s = solver.findOptimalSolution(Objective.MAXIMIZE, p.getVar("P"));
		if (s == null)
			p.log("Unable to derive a solution.");
		else {
			p.log("*** Solution ***");
			p.log("Maximum Profit = " + s.getValue("P"));
			p.log("Gold (" + s.getValue("G") + ")  Silver ("
					+ s.getValue("S") + ") Bronze(" + s.getValue("B") + ")");
		}
		// --- Display Stats Log (optional)
		solver.logStats();

	}

	public static void main(String[] args) {

		Knapsack theProblem = new Knapsack();
		theProblem.define();
		theProblem.solve();

	}

}
